#include <QtGui>
#include <QtNetwork>

class Window : public QMainWindow
{
	Q_OBJECT

public:
	Window( QWidget* parent = 0 )
		: QMainWindow( parent )
	{
		login = new QLineEdit( this );
		password = new QLineEdit( this );
		host = new QLineEdit( this );
		port = new QSpinBox( this );
		port->setMaximum( 65535 );
		bt = new QPushButton( tr( "Connect" ), this );
		bt->setCheckable( true );
		
		QHBoxLayout* hl1 = new QHBoxLayout;
		hl1->addWidget( new QLabel( tr( "Login" ) ) );
		hl1->addWidget( login );
		hl1->addWidget( new QLabel( tr( "Password" ) ) );
		hl1->addWidget( password );
		
		QHBoxLayout* hl2 = new QHBoxLayout;
		hl2->addWidget( new QLabel( tr( "Host" ) ) );
		hl2->addWidget( host );
		hl2->addWidget( new QLabel( tr( "Port" ) ) );
		hl2->addWidget( port );
		hl2->addWidget( bt );
		
		pte = new QPlainTextEdit( this );
		pte->setTabStopWidth( 40 );
	
		QWidget* widget = new QWidget( this );
		QVBoxLayout* vl = new QVBoxLayout( widget );
		vl->addLayout( hl1 );
		vl->addLayout( hl2 );
		vl->addWidget( pte );
		
		setCentralWidget( widget );
		
		ftp.setTransferMode( QFtp::Active );
		
		connect( bt, SIGNAL( clicked() ), this, SLOT( connectFtp() ) );
		
		connect( &ftp, SIGNAL( commandFinished( int, bool ) ), this, SLOT( commandFinished( int, bool ) ) );
		connect( &ftp, SIGNAL( commandStarted( int ) ), this, SLOT( commandStarted( int ) ) );
		connect( &ftp, SIGNAL( dataTransferProgress( qint64, qint64 ) ), this, SLOT( dataTransferProgress( qint64, qint64 ) ) );
		connect( &ftp, SIGNAL( done( bool ) ), this, SLOT( done( bool ) ) );
		connect( &ftp, SIGNAL( listInfo( const QUrlInfo& ) ), this, SLOT( listInfo( const QUrlInfo& ) ) );
		connect( &ftp, SIGNAL( rawCommandReply( int, const QString& ) ), this, SLOT( rawCommandReply( int, const QString& ) ) );
		connect( &ftp, SIGNAL( readyRead() ), this, SLOT( readyRead() ) );
		connect( &ftp, SIGNAL( stateChanged( int ) ), this, SLOT( stateChanged( int ) ) );
	}

protected:
	QLineEdit* login;
	QLineEdit* password;
	QLineEdit* host;
	QSpinBox* port;
	QPushButton* bt;
	QPlainTextEdit* pte;
	QFtp ftp;
	QSet<int> connectQueries;
	QSet<int> closeQueries;
	QSet<int> loginQueries;
	QSet<int> queries;
	int step;

protected slots:
	void connectFtp()
	{
		if ( ftp.state() != QFtp::Unconnected ) {
			ftp.clearPendingCommands();
			ftp.abort();
			closeQueries << ftp.close();
		}
		
		if ( bt->isChecked() ) {
			step = 0;
			connectQueries << ftp.connectToHost( host->text(), port->value() );
			loginQueries << ftp.login( login->text(), password->text() );
			pte->appendPlainText( tr( "Connection..." ) );
		}
	}
	
	void commandFinished( int id, bool error )
	{
		QString text;
		
		if ( connectQueries.contains( id ) ) {
			text = tr( "(connect)" );
		}
		else if ( closeQueries.contains( id ) ) {
			text = tr( "(close)" );
		}
		else if ( loginQueries.contains( id ) ) {
			text = tr( "(login)" );
		}
		else if ( queries.contains( id ) ) {
			text = tr( "(query)" );
		}
		else {
			text = tr( "(unknow)" );
		}
		
		pte->appendPlainText( tr( "** Query %1 finished, error %2 %3 (%4)" ).arg( id ).arg( error ).arg( text ).arg( ftp.errorString().simplified() ) );
	}
	
	void commandStarted( int id )
	{
		QString text;
		
		if ( connectQueries.contains( id ) ) {
			text = tr( "(connect)" );
		}
		else if ( closeQueries.contains( id ) ) {
			text = tr( "(close)" );
		}
		else if ( loginQueries.contains( id ) ) {
			text = tr( "(login)");
		}
		else if ( queries.contains( id ) ) {
			text = tr( "(query)" );
		}
		else {
			text = tr( "(unknow)" );
		}
		
		pte->appendPlainText( tr( "** Query %1 started %3" ).arg( id ).arg( text ) );
	}
	
	void dataTransferProgress( qint64 done, qint64 total )
	{
		Q_UNUSED( done );
		Q_UNUSED( total );
		pte->appendPlainText( tr( "** Transfer progress..." ) );
	}
	
	void done( bool error )
	{
		pte->appendPlainText( tr( "** Done, error %1 (%2)" ).arg( error ).arg( ftp.errorString().simplified() ) );
		
		static int count = 0; // bypass first query which is when changing mode to QFtp::Active.
		
		if ( count > 0 ) {
			const QByteArray content = "Intuisphere test content";
			const QString folder = "intuisphere.test.dir.can.be.removed.safety";
			const QString file = "intuisphere.test.can.be.removed.safety.txt";
			
			switch ( step )
			{
				case 0: {
					const int id = ftp.mkdir( folder );
					pte->appendPlainText( QString( "** Mkdir folder intuisphere.* %1" ).arg( id ) );
					queries << id;
					break;
				}
				case 1: {
					const int id = ftp.cd( folder );
					pte->appendPlainText( QString( "** Cd folder intuisphere.* %1" ).arg( id ) );
					queries << id;
					break;
				}
				case 2: {
					const int id = ftp.put( content, file, QFtp::Ascii );
					pte->appendPlainText( QString( "** Putting file intuisphere.*.txt %1" ).arg( id ) );
					queries << id;
					break;
				}
				case 3: {
					const int id = ftp.cd( ".." );
					pte->appendPlainText( QString( "** Cd folder .. %1" ).arg( id ) );
					queries << id;
					break;
				}
				case 4: {
					const int id = ftp.put( content, file, QFtp::Ascii );
					pte->appendPlainText( QString( "** Putting file intuisphere.*.txt %1" ).arg( id ) );
					queries << id;
					break;
				}
				case 5: {
					const int id = ftp.remove( file );
					pte->appendPlainText( QString( "** Removing file intuisphere.*.txt %1" ).arg( id ) );
					queries << id;
					break;
				}
				case 6: {
					const int id = ftp.remove( QString( "%1/%2" ).arg( folder ).arg( file ) );
					pte->appendPlainText( QString( "** Removing file intuisphere.*.txt %1" ).arg( id ) );
					queries << id;
					break;
				}
				case 7: {
					const int id = ftp.remove( folder );
					pte->appendPlainText( QString( "** Removing folder intuisphere.* %1" ).arg( id ) );
					queries << id;
					break;
				}
			}
			
			step++;
		}
		
		count++;
	}
	
	void listInfo( const QUrlInfo& i )
	{
		QString text = QString( "%1 (%2) %3bytes" )
			.arg( i.name() )
			.arg( i.isDir() ? tr( "Directory" ) : tr( "File" ) )
			.arg( i.size() );
		
		pte->appendPlainText( tr( "\t* List info: %1" ).arg( text ) );
	}
	
	void rawCommandReply( int replyCode, const QString& detail )
	{
		pte->appendPlainText( tr( "** Raw reply, code: %1, detail: %2" ).arg( replyCode ).arg( detail.simplified() ) );
	}
	
	void readyRead()
	{
		pte->appendPlainText( tr( "** Ready read" ) );
	}
	
	void stateChanged( int state )
	{
		QString text;
		
		switch ( state )
		{
			case QFtp::Unconnected:
				text = tr( "Unconnected" );
				bt->setChecked( false );
				break;
			case QFtp::HostLookup:
				text = tr( "HostLookup" );
				break;
			case QFtp::Connecting:
				text = tr( "Connecting" );
				break;
			case QFtp::Connected:
				text = tr( "Connected" );
				break;
			case QFtp::LoggedIn:
				text = tr( "LoggedIn" );
				queries << ftp.list();
				break;
			case QFtp::Closing:
				text = tr( "Closing" );
				break;
		}
		
		pte->appendPlainText( tr( "** State changed %1 (%2)" ).arg( state ).arg( text ) );
	}
	
};

int main( int argc, char** argv )
{
	// create application
	QApplication app( argc, argv );

	// init application
	app.setApplicationName( "ftp" );

	Window w;
	w.setWindowTitle( "ftp" );
	w.show();

	// connection
	QObject::connect( &app, SIGNAL( lastWindowClosed() ), &app, SLOT( quit() ) );

	// start application
	return app.exec();
}

#include "main.moc"
